/* global Packages, sc */


let objects = require('common/objects');
let metadata = require('filingassistant/metadata');
let transaction = require('common/transaction');
let templates = require('common/templates');
let CSVParser = require('common/csv');
let beans = require('common/beans');

let inbox = objects.tryFind('/agorum/roi/Files/agorum core open example import csv/Inbox');
let archiveFolder = objects.tryFind('/agorum/roi/Files/agorum core open example import csv/Storage/Archive');
let errorFolder = objects.tryFind('/agorum/roi/Files/agorum core open example import csv/Storage/Error');
let protocolsFolder = objects.tryFind('/agorum/roi/Files/agorum core open example import csv/Storage/Protocols');

if (inbox && archiveFolder && errorFolder && protocolsFolder) {
  // Alle Ordner sind vorhanden
  
  let query = 'inpath:' + inbox.ID + ' classname:fileobject';   // Nur Dateien holen, sonst nichts

  objects.query(query).iterate('uuid', data => {
    let item = objects.tryFind(data.uuid);
    if (item) {

      if (new Date().getTime() - item.lastModifyDate.getTime() < 15000) {
        // skip files, when not older than 15 seconds
        return;
      }

      let ok;

      try {
        transaction(function() {
          ok = agorum_open_example_import_csv_SetMetadatenFromCSV(inbox, item);
        });
      }
      catch (err) {

        console.log('ERROR', err);
        // der Fehler wird rausgeschrieben

        transaction(function() {
          let fehler = objects.create('file', {
            name: item.name + '.err.txt',
            target: errorFolder
          });
          fehler.contentString = err;
        });



        java.lang.System.err.println('agorum_open_example_import_csv_eingang.js: Error procesing object ' + item);

        if (err.javaException) {
          err.javaException.printStackTrace();
        }
        else {
          java.lang.System.err.println('' + err);
        }
      }

      if (!ok) {
        transaction(function() {
          agorum_open_example_import_csv_MoveError(item);
        });
      }
    }
  });

}

function agorum_open_example_import_csv_MoveError(file) {
  // Ist nur ein Fehler, wenn es sich um eine csv handelt
  // Diese wurde dann nicht verarbeitet, sondern hat einen Fehler geliefert
  // z.B. weil in der csv ein fehlerhaftest Datumsformat oder sowas ist
  
  if ((file.nameExtension || '').toLowerCase() !== 'csv') return;
  
  // Jetzt prüfen ob eine entsprechende PDF im selben Verzeichnis ist
  let parent = file.parents()[0];
  let pdfFile = parent.getItem(file.baseName + '.pdf');
  
  if (pdfFile) {
    objects.link(pdfFile, errorFolder);
    objects.unlink(pdfFile, [ parent ]);
    pdfFile.acl = errorFolder.acl;
  }
  
  if (file) {
    objects.link(file, errorFolder);
    objects.unlink(file, [ parent ]);
    file.acl = errorFolder.acl;
  }
}

function agorum_open_example_import_csv_SetMetadatenFromCSV(folder, file) {
  
  
  
  if ((file.nameExtension || '').toLowerCase() === 'csv') {
    console.log('agorum_open_example_import_csv_SetMetadatenFromCSV', file);
    // Jetzt prüfen ob eine entsprechende PDF im selben Verzeichnis ist
    let parent = folder; //file.parents()[0];
    let pdfFile = parent.getItem(file.baseName + '.pdf');
    if (pdfFile) {
      // PDF-File ist auch hier
      
      // Ist eine CSV
      let result = getCSV(file);
      //
      // jetzt die Metadaten auslesen und auf die PDF setzen
      // 
      let data = {};
      
      if (result[0]['Formular-ID']) { 
        data.agorum_open_example_import_csv_formular_id = result[0]['Formular-ID'];
        
        // zerlegen der Formular-ID
        let fidArr = data.agorum_open_example_import_csv_formular_id.split('.');
        data.agorum_open_example_import_csv_fid_system_name        = fidArr[0];
        data.agorum_open_example_import_csv_fid_company           = fidArr[1];        
        data.agorum_open_example_import_csv_fid_archive_nr  = fidArr[2];
        data.agorum_open_example_import_csv_fid_format            = fidArr[3];     

      }
      if (result[0]['Beleg-Title']    ) { 
        
        // Beleg-Titel (eindeutiger Belegname)
        
        data.agorum_open_example_import_csv_beleg_title     = result[0]['Beleg-Title'];
      }
      if (result[0]['Beleg-Nr']) { 
        data.agorum_open_example_import_csv_beleg_nr        = result[0]['Beleg-Nr'];
      }
      
      if (result[0]['Auftrags-Nr']) { 
        data.agorum_open_example_import_csv_auftrags_nr    = result[0]['Auftrags-Nr'];
      }

      //Fahrzeugnummer
      if (result[0].Fahrzeugnummer) { 
        data.agorum_open_example_import_csv_fahrzeug_nummer    = result[0].Fahrzeugnummer;
      }
      if (result[0]['Beleg-Zuordnung']) { 
        data.agorum_open_example_import_csv_beleg_zuordnung = result[0]['Beleg-Zuordnung'];
      }
      if (result[0]['Filiale-Nr']     ) { 
        data.agorum_open_example_import_csv_filiale_nr      = result[0]['Filiale-Nr'];
      }
      if (result[0]['Beleg-Datum']    ) { 
        data.agorum_open_example_import_csv_beleg_datum     = getDateFromString(result[0]['Beleg-Datum'],'dd.MM.yyyy');       // DATUM
      }
      if (result[0].Leistungsdatum ) { 
        data.agorum_open_example_import_csv_leistungs_datum = getDateFromString(result[0].Leistungsdatum,'dd.MM.yyyy');    // DATUM
      }
      if (result[0]['Kunden-Nr']      ) { 
        data.agorum_open_example_import_csv_kunden_nr       = result[0]['Kunden-Nr'];
      }        
      if (result[0].Kennzeichen    ) { 
        data.agorum_open_example_import_csv_kennzeichen    = result[0].Kennzeichen;
      }
      if (result[0].Erstzulassung  ) { 
        data.agorum_open_example_import_csv_erstzulassung  = getDateFromString(result[0].Erstzulassung,'dd.MM.yyyy');     // DATUM
      }        
      if (result[0]['Motor-Nr']       ) { 
        data.agorum_open_example_import_csv_motornr        = result[0]['Motor-Nr'];
      }        
      if (result[0].Herstellercode ) { 
        data.agorum_open_example_import_csv_herstellercode = result[0].Herstellercode;
      }        
      if (result[0]['Fahrgestell-Nr'] ) { 
        data.agorum_open_example_import_csv_fahrgestell_nr  = result[0]['Fahrgestell-Nr'];
      }
      if (result[0].PLZ            ) { 
        data.agorum_open_example_import_csv_plz            = result[0].PLZ;
      }        
      if (result[0].Ort            ) { 
        data.agorum_open_example_import_csv_ort            = result[0].Ort;
      }       
      if (result[0].Rechnungsbetrag) { 
        data.agorum_open_example_import_csv_rechnungsbetrag= getDoubleFromString(result[0].Rechnungsbetrag);   // Double
      }       
      
      
      // Collection noch setzen, unter der die Metadaten zusammengefasst sind
      
      data.ag_metadata_collection = 'agorum_open_example_import_csv_voucher';
      
      // Jetzt die Metadaten auf die PDF setzen      
                  
      metadata(data).save(pdfFile);      
      
      let subPath = templates.fill('${d:yyyy}/${d:MM}', {
        d: data.agorum_open_example_import_csv_beleg_datum || data.agorum_open_example_import_csv_leistungs_datum || new Date()
      });
                  
      // Jetzt wird die PDF verschoben
      objects.link(pdfFile, archiveFolder.createPath(subPath));
      objects.unlink(pdfFile, [ parent ]);
      
      // PDF-Datei mit der csv verknüpfen
      pdfFile.setAttachments([file]);      
      
      // csv - Protokollieren
      objects.link(file, protocolsFolder.createPath(subPath));
      objects.unlink(file, [ parent ]);      
      
      return true;
    }
  }
  
  return false;
}



// Hier die Regeln aus der CSV auslesen
//
function getCSV(csv) {
  let result = CSVParser.read(csv,{
    delimiter: ';',
    header: true,
    encoding: 'UTF-8',
  });
  
  return result;
}

function getDateFromString(str,format) {
  
  let SimpleDateFormat = Packages.java.text.SimpleDateFormat;
  let sdf = new SimpleDateFormat(format);
  let date = sdf.parse(str);
  return date;
}

function getDoubleFromString(str) {
  str =  str.replace(/\./g, "");
  str =  str.replace(/\,/g, ".");
  return Packages.java.lang.Double.parseDouble(str);
}

